﻿using UnityEngine;
using System.Collections;

public class Boss : MonoBehaviour {

	[SerializeField] private Transform playerTransform;
	[SerializeField] private int maxHitPoints = 50;
	[SerializeField] private int angerHitPoints = 20;
	private int hitPoints;
	[SerializeField] private float movementSpeed = 5;

	[SerializeField] private Animator myAnimator;

	[SerializeField] private int[] possibleAttacks;

	private bool followsPlayer = true;
	private bool canWalk = true;
	

	public void SpeedUp()
	{
		movementSpeed *= 10;
	}

	public void SpeedDown()
	{
		movementSpeed /= 10;
	}

	public void ChooseNextAttack()
	{
		myAnimator.SetInteger("Attack", possibleAttacks[Random.Range(0,possibleAttacks.Length)]);
	}

	private void Awake()
	{
		hitPoints = maxHitPoints;
		StartCoroutine(CheckAngryMode());
	}

	private IEnumerator CheckAngryMode()
	{
		bool isntAngry = true;

		do
		{
			isntAngry = hitPoints > angerHitPoints;
			yield return new WaitForSeconds(2);
		} while (isntAngry);

		myAnimator.speed *= 1.5f;
		movementSpeed    *= 1.5f;
	}

	private Vector3 auxVector3;
	private Vector3 distanceToPlayer;
	private void Update()
	{
		rigidbody.velocity = Vector3.right * 0.1f;
		if (followsPlayer) {
						auxVector3 = transform.rotation.eulerAngles;
						distanceToPlayer = playerTransform.position - transform.position;
						auxVector3.z = 360 - Mathf.Rad2Deg * Mathf.Atan2 (distanceToPlayer.x, distanceToPlayer.y);
						transform.rotation = Quaternion.Euler (auxVector3);
				} else
						Debug.Log (transform.rotation);
		if (canWalk) {
			rigidbody.velocity = transform.up * movementSpeed;
		}
	}

	public void ResetAttackState()
	{
		myAnimator.SetInteger("Attack", 0);
	}

	public void StopFollowing()
	{
		Debug.Log ("algo");
		followsPlayer = false;
	}

	public void StartFollowing()
	{
		followsPlayer = true;
	}

	public void StopWalking()
	{
		canWalk = false;
	}

	
	public void StartWalking()
	{
		canWalk = true;
	}

	public void SpeedUpAttack()
	{
		myAnimator.speed *= 2;
	}

	public void SlowDownAttack()
	{
		myAnimator.speed /= 4;
	}

	private void OnTriggerEnter(Collider collider)
	{
		if (collider.tag.Equals ("DamagingSpell")) {
			InvocationSpell spell = collider.GetComponent<InvocationSpell> ();
			Debug.Log(spell);
			if (spell.friendly) {
				hitPoints -= Mathf.CeilToInt(spell.spellPower);
				if (hitPoints <= 0) GameObject.Destroy(this.gameObject);
			}
		}
	}
}