﻿using UnityEngine;
using System.Collections;

public class PlayerController : MonoBehaviour {

	[SerializeField] private float walkSpeed = 5;
	[SerializeField] private GameObject attackPrefab;

	[SerializeField] private Transform spriteTransform;
	
	[SerializeField] private int maxHitPoints = 100;
	                 private int hitPoints;

	[SerializeField] private float attackSpeed = 0.5f;
	private bool canAttack = true;
	[SerializeField] private CooldownController   attackCooldownController;
	[SerializeField] private CooldownController[] magicCooldownController;
	private bool[] magicAvailable;

	private Vector3 forward;
	private Animator myAnimator;

	[SerializeField] private float mercyInvincibilityTime = 0.5f;
	private bool canBeDamaged = true;

	static private PlayerController myInstance;
	static public  float rotation
	{
		get
		{
			if (myInstance == null) return 0;

			Debug.Log("Player rotation = " + Vector2.Angle(Vector2.zero, myInstance.forward));

			return Vector2.Angle(Vector2.zero, myInstance.forward);
		}
	}

	static public float hpRatio
	{
		get { return (float)myInstance.hitPoints / myInstance.maxHitPoints; }
	}

	static public void Heal (int amount)
	{
		Debug.Log("Healing HP by " + amount);
		if (myInstance == null || amount < 0 || myInstance.hitPoints == myInstance.maxHitPoints) return;
		myInstance.hitPoints += amount;
		if (myInstance.hitPoints > myInstance.maxHitPoints) myInstance.hitPoints = myInstance.maxHitPoints;
	}

	static public Vector3 position {
		get {
			if (myInstance == null) return Vector3.zero;
			return myInstance.transform.position;
		}
	}

	static public Vector3 forwardVector {
		get {
			if (myInstance == null) return Vector3.right;
			return myInstance.forward;
		}
	}

	public void Hurt(int damage)
	{
		if (damage < 0 || myInstance.hitPoints == 0) return;
		myInstance.hitPoints -= damage;
		if (hitPoints <= 0)
		{
			hitPoints = 0;
			Debug.Log ("Game over!");
		}
	}

	// Use this for initialization
	void Awake ()
	{
		myInstance = this;

		myAnimator = this.GetComponent<Animator>();

		hitPoints = maxHitPoints;

		int magicCount = magicCooldownController.Length;
		magicAvailable = new bool[magicCount];
		for (int i = 0; i < magicCount; i++)
		{
			magicAvailable[i] = true;
		}
	}
	
	// Update is called once per frame
	void Update ()
	{
		rigidbody.velocity = Vector3.right * 0.1f;
		if (Input.GetKeyDown (KeyCode.F))
		{
			hitPoints -= 10;
		}

		float ataque;

		for (int i = 0; i < magicCooldownController.Length; i++) {
			if (magicAvailable[i])
			{
				ataque = Input.GetAxis ("Spell" + (i + 1));
				if (ataque > 0) {
					myAnimator.SetBool("IsCasting", true);
					magicAvailable[i] = false;
					magicCooldownController[i].UseEffect(EnableMagic, i);
					rigidbody.velocity = Vector3.zero;
//					GameObject.Instantiate(attackPrefab, transform.position, Quaternion.LookRotation(forward));
				}
			}
		}

		if (!canAttack) return;

		ataque = Input.GetAxis ("Attack");
		if (ataque > 0)
		{
			myAnimator.SetBool("IsAttacking", true);
//			canAttack = false;
			attackCooldownController.UseEffect(attackSpeed, EnableAttack);
			GameObject.Instantiate(attackPrefab, transform.position + forward, Quaternion.identity);
			rigidbody.velocity = Vector3.zero;
//			GameObject.Instantiate(attackPrefab, transform.position, Quaternion.LookRotation(forward));
		}

		float hSpeed = Input.GetAxis("Horizontal");
		float vSpeed = Input.GetAxis("Vertical");
		if (hSpeed == 0 && vSpeed == 0)
		{
			myAnimator.SetBool("IsWalking", false);
			return;
		}

		myAnimator.SetBool("IsWalking", true);
//		if (Mathf.Abs (hSpeed) > Mathf.Abs (vSpeed))
//		{
		forward = (Vector3.right * hSpeed + Vector3.up * vSpeed).normalized;
		Vector3 bla = spriteTransform.rotation.eulerAngles;
		bla.z = 360 - Mathf.Rad2Deg * Mathf.Atan2(forward.x, forward.y);
		spriteTransform.rotation = Quaternion.Euler(bla);
		if (canAttack) rigidbody.velocity = forward * walkSpeed;
//		}
	}

	private void EnableAttack()
	{
//		canAttack = true;
	}

	private void EnableMagic(int magicIndex)
	{
		magicAvailable[Mathf.Clamp(magicIndex, 0, magicAvailable.Length)] = true;
	}

	private void OnTriggerEnter(Collider collider)
	{
		Debug.Log ("Trigger enter!");
		if (canBeDamaged && collider.tag.Equals ("DamagingSpell")) {
			InvocationSpell spell = collider.GetComponent<InvocationSpell> ();
			Debug.Log(spell);
			if (!spell.friendly) {
				hitPoints -= Mathf.CeilToInt (spell.spellPower);
				StartCoroutine (GiveMercyInvincibility ());	
				if (hitPoints <= 0) Application.LoadLevel ("Mainmenu");
			}
		}
	}

	private void OnCollisionEnter(Collision collision)
	{
		Debug.Log ("Trigger enter!");
		if (canBeDamaged && collision.collider.tag.Equals ("DamagingSpell")) {
			InvocationSpell spell = collision.collider.GetComponent<InvocationSpell> ();
			if (!spell.friendly) {
				hitPoints -= Mathf.CeilToInt (spell.spellPower);
				StartCoroutine (GiveMercyInvincibility ());	
				if (hitPoints <= 0) Application.LoadLevel ("Mainmenu");
			}
		}
	}

	private IEnumerator GiveMercyInvincibility()
	{
		canBeDamaged = false;
		yield return new WaitForSeconds(mercyInvincibilityTime);
		canBeDamaged = true;
	}

	private void Attack(){
		canAttack = false;
	}

	private void EndAttack(){
		canAttack = true;
	}

//	private void OnCollisionEnter(Collision collision)
//	{
//		transform.position = collision.contacts[0].point;
//	}
}
